import os
from cgi import parse_qs, escape
from emod_api import peek_camp as pc
import site
import json


"""
1) Add a form that lets user fill out any/all fields of a CCDL-based event.
2) Render that back in the form for editing, plus render as CCDL.
3) Render as json.
4) Ingest a CCDL campaign, letting user select events to edit.
5) Render graphically.
"""

# This function will go into emod_api.
def params_to_ccdl( start_day, duration=None, reps=None, gap=None, nodes=None, frac=None, sex=None, minage=None, maxage=None, ips=None, signal=None, iv_name=None, payload=None, delay=None ):
    day = f"{start_day}"
    if reps and reps != "None":
        day += f"(x{reps}/_{gap})"
    elif duration and duration != "None":
        day += f"-{start_day+duration}"

    nodes = nodes
    #who = f"{frac}%"
    who = "STEERED" if type(frac) is str else f"{float(frac)*100}%"
    if "ale" in sex:
        who += f"/{sex}"
    if minage > 0:
        who += f"/>{minage}"
    if maxage < 120*365.:
        who += f"/<{maxage}"
    if ips and ips != "" and ips != "None":
        who += f"/{ips}"

    if payload:
        iv_name = f"{iv_name}({payload})"
    else:
        iv_name = f"{iv_name}"
    if signal:
        iv_name = f"{signal}->{iv_name}"

    ccdl = f"{day} :: {nodes} :: {who} :: {iv_name}"
    return ccdl

def application(environ, start_response):
    status = '200 OK'
    #output = b'Hooray, mod_wsgi is working'
    output = b"<html><body><H1>CCDL Demo</H1>"
    # Process input...
    d = parse_qs( environ["QUERY_STRING"] )
    start = 1
    duration = -1
    reps = 1
    period = None

    where = "All"

    #who = "100%"
    coverage = "100%"
    sex = "Both"
    min_age = 0
    max_age = 125
    ips = ""

    # what = "NewInfection->SimpleVaccine(0.95)"
    trigger = "None" # will be from dropdown
    delay = "None"
    iv = "SimpleVaccine" # will be from dropdown
    iv_payload = "None"

    if len( d.keys() ) > 0:
        #when = d["when"][0]
        # when is either:
        # * Start_Day
        # * Start_Day-Stop_Day
        # * Start_Day(xreps/_period)
        #start=float(when.split(["-"][0].split("(")[0]))
        start=float(d["start"][0])
        if "duration" in d.keys():
            duration=float(d["duration"][0])
        if "reps" in d.keys():
            reps=float(d["reps"][0])
        if "period" in d.keys() and d["period"][0] != "None":
            period=float(d["period"][0])

        where = d["where"][0]

        #who = d["who"][0]
        if "coverage" in d.keys():
            coverage =d["coverage"][0] 
            if "%" in coverage:
                coverage = float(coverage.split( "%" )[0])/100.
            else:
                coverage = float(coverage)
        if "sex" in d.keys():
            sex=d["sex"][0]
        if "min_age" in d.keys():
            min_age=float(d["min_age"][0])
        if "max_age" in d.keys():
            max_age=float(d["max_age"][0])
        if "ips" in d.keys():
            ips=d["ips"][0]

        #what = d["what"][0] 
        if "trigger" in d.keys():
            trigger=d["trigger"][0]
        if "delay" in d.keys():
            delay=d["delay"][0]
        if "iv" in d.keys():
            iv=d["iv"][0]
        if "iv_payload" in d.keys():
            iv_payload=d["iv_payload"][0]

        ccdl = params_to_ccdl( start_day=start, duration=duration, reps=reps, gap=period, nodes=where, frac=coverage, sex=sex, minage=min_age, maxage=max_age, ips=ips, signal=trigger, delay=delay, iv_name=iv, payload=iv_payload )
        # call emod_api.peek_campaign.encode_from_data( ccdl )

    output += f"""
        <H2>Editor</H2>
	<form action="/test_wsgi">
	  <label for="start"><b>When</b></label><br>
          <label for="start">Start_Day: </label>
	  <input type="text" id="start" name="start" value="{start}">
          <label for="duration">Duration (Opt): </label>
	  <input type="text" id="duration" name="duration" value="{duration}">
          <label for="reps">Repetitions (Opt): </label>
	  <input type="text" id="reps" name="reps" value="{reps}">
          <label for="period">Rep Interval (Opt): </label>
	  <input type="text" id="period" name="period" value="{period}"><br><br>

	  <label for="where"><b>Where</b></label><br>
	  <input type="text" id="where" name="where" value={where}><br><br>

	  <label for="lname"><b>Who</b></label><br>
          <label for="coverage">Coverage: </label>
	  <input type="text" id="coverage" name="coverage" value={coverage}>
          <label for="sex">Targeted Sex (Opt): </label>
	  <select id="sex" name="sex" value={sex}>
              <option value="Both">Both</option>
              <option value="Male">Male</option>
              <option value="Female">Female</option>
          </select>
          <label for="min_age">Targeted Min Age (Opt): </label>
	  <input type="text" id="min_age" name="min_age" value={min_age}>
          <label for="max_age">Targeted Max Age (Opt): </label>
	  <input type="text" id="max_age" name="max_age" value={max_age}>
          <label for="ips">Targeted Individual Properties (Opt): </label>
	  <input type="text" id="ips" name="ips" value={ips}><br><br>

	  <label for="lname"><b>What</b></label><br>
          <label for="trigger">Trigger (Opt): </label>
	  <input type="text" id="trigger" name="trigger" value={escape(trigger)}>
          <label for="trigger">Delay (Opt): </label>
	  <input type="text" id="delay" name="delay" value={escape(delay)}>
          <label for="trigger">Intervention: </label>
          """.encode( "ascii" )
    output += f"""
	  <select id="iv" name="iv" value={escape(iv)}>
              <option value="OutbreakIndividual">OutbreakIndividual</option>
              <option value="SimpleVaccine">SimpleVaccine</option>
              <option value="PropertyValueChanger">PropertyValueChanger</option>
              <option value="HealthSeekingBehavior">HealthSeekingBehavior</option>
              <option value="BroadcastEvent">BroadcastEvent</option>
              <option value="TBHIVConfigurableDrug">TBHIVConfigurableDrug</option>
              <option value="DiagnosticTreatNeg">DiagnosticTreatNeg</option>
              <option value="AntiMalarialDrug">AntiMalarialDrug</option>
          </select>
          <label for="trigger">Intervention Param (Context-Sensitive): </label>
	  <input type="text" id="iv_payload" name="iv_payload" value={iv_payload}><br><br>
	  <input type="submit" value="Submit">
	</form><hr>
        """.encode( "ascii" )
    output += "<H2>CCDL Format</H2>".encode( "ascii" )
    output += ccdl.encode( "ascii" ) + "<hr>".encode( "ascii" )

    with open( "/var/opt/idm/apache/camp.ccdl", "w" ) as ccdl_fp: # does nothing!? :(
        ccdl_fp.write( ccdl )

    import emod_api.campaign as camp
    from emodpy_hiv.camp_creator import create_campaign_from_concise as ccfc
    import emodpy_hiv.camp_creator as creator
    outfile = "/var/opt/idm/test_campaign.json"
    camp.set_schema( "/var/www/html/schema.json" )
    camp.unsafe = True
    ccfc( "/var/opt/idm/apache/camp.ccdl", outfile=outfile )

    with open( outfile, "r" ) as camp_json_fp:
        camp_json = json.load( camp_json_fp )
        output += "<H2>JSON Format</H2>".encode( "ascii" )
        output += "<pre>".encode( "ascii" ) + json.dumps( camp_json, indent=4, sort_keys=True ).encode( "ascii" ) + "</pre>".encode( "ascii" )

    output += f"DEBUG: {site.getsitepackages()}".encode( "ascii" )
    output += f"\nDEBUG: {os.getcwd()}".encode( "ascii" )
    output += """
        </body>
	</html>
    """.encode( "ascii" )
 
    response_headers = [('Content-type', 'text/html'),
                        ('Content-Length', str(len(output)))]
    start_response(status, response_headers)
 

    return [output]
